from typing import Any, Dict


class BaseCluster:
    __prefix__: str = ...
    __ttl__: int = ...
    ENABLE_CACHING: bool = True  # Default is ON; override per class if needed
    
    def __init__(self, inherited_params: Dict[str, Any] = None, _parent=None, redis_client=None):...
    def get_full_prefix(self):...
    def with_params(self, **params):...
    def describe(self):...
    
    async def clear(self) -> None:...
    async def subkeys(self) -> None:...
